unit Parsebuf;
{
	For parsing a buffer (null-terminated) containing data field(s) delimited
  by a Separator character.  Returns each field individually as pointer to
  null-terminated buffer.

  Defaults to TAB as Separator character, though you may set to any character.

  .................Example usage:.......................

  var
  	pb : TParsebuf;
    FieldPtr : PChar;
  begin
  	pb:= TParseBuf.Create(Application);
  	with pb do try
	    Separator := '~';        				..to define separator other than TAB
  	  SetBuf(SomeBufferContainingFields);
    	while NextField do begin
    		FieldPtr := GetField;
	      ...do something with FieldValue...
  	  end;
	  finally
  		Free;
	  end;
  end;

  ......................................................

  SomeBufferContainingFields, above, may be a String, if you add a null
  terminator to the string, using a function like:

	procedure AddNullTo(var ss: String);
	begin
		if length(ss) > 254 then
  		raise EStringTooLong.Create('String too long...cannot add terminating null')
	  else
  		ss[length(ss)+1] := #0;
	end;

  So for a string, the SetBuf call in the example above would be:

	  AddNullTo(mystring);         ...add terminating null
    SetBuf(@mystring[1]);        ...pass pointer to first char of string

  ......................................................

  Includes some miscellaneous routines which provide access to data I needed
  for monitoring purposes.
}

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs;

type
  TParseBuf = class(TComponent)
  private
	  bufptr: PChar;								{ptr into buffer}
    fieldbufsize: word;       {size of the FieldBuf buffer}
		fieldbuf: PChar;		{buffer individual fields are placed}
    		{published}
    FSeparator: char;
    FFieldCount:	word;       {no. fields in the buffer}
  public
		constructor Create(AOwner: TComponent);
		destructor Destroy;
		procedure SetFieldBufSize(newsize: word);
		procedure SetBuf(buf: PChar);
		function CountFields: Word;
		function CurrBufStart: PChar;
		function NextField: boolean;
		function GetField: PChar;
  published
    property Separator: char read FSeparator write FSeparator default #9; {tab}
    property FieldCount: word read FFieldCount write FFieldCount;
  end;

procedure Register;

implementation

const
	DEFAULTFIELDBUFSIZE = 256;	{Set one larger than largest expected field}

constructor TParseBuf.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  fieldbufsize := DEFAULTFIELDBUFSIZE;
  GetMem(fieldbuf, fieldbufsize);
end;


destructor TParseBuf.Destroy;
begin
  FreeMem(fieldbuf, fieldbufsize);
  inherited Destroy;
end;


procedure TParseBuf.SetFieldBufSize(newsize: word);
{	Necessary only if you want to allocate field buffer of size different
	from the default.}
begin
	if newsize > 0 then begin
	  FreeMem(fieldbuf, fieldbufsize);
	  fieldbufsize := newsize;
	  GetMem(fieldbuf, fieldbufsize);
  end;
end;


procedure TParseBuf.SetBuf(buf: PChar);
{ Point to passed buffer to parse...buf MUST be null-terminated string.}
begin
 	bufptr := buf;
 	FFieldCount := CountFields;
end;


function TParseBuf.CountFields: Word;
{	Count fields in buffer. }
var
  fcount: Word;				{field count}
  ps: PChar;          {ptr to next separator}
begin
	if bufptr^ = #0 then
    Result := 0
  else begin
    fcount := 1;									{count first field (before delimiter)}
    ps := StrScan(bufptr, FSeparator);
    while ps<>nil do begin
      Inc(fcount);
      Inc(ps);
      ps := StrScan(ps, FSeparator);
    end;
    Result := fcount;
  end;
end;


function TParseBuf.CurrBufStart: PChar;
{Ptr to current start of buffer}
begin
	Result := bufptr;
end;


function TParseBuf.NextField: boolean;
{ Retrieve next field from buffer, write to FieldBuf.
  Leaves bufptr pointing to beginning of field following the one retrieved.
  Return: 		TRUE if field retrieved, FALSE if not (end of buffer)
}
label done, reallydone;
var
  p: PChar;           {ptr into buf}
  fp: PChar;					{ptr into field bufr}
begin
  fp := @FieldBuf[0];

  if (bufptr = nil) or
		 (bufptr^ = #0) then begin         	{no more fields}
    Result := False;
    goto reallydone;
  end;
	Result := True;

			{Point to next delimiter}
 	p := StrScan(bufptr, FSeparator);
  if p=nil then            	{no more delims...point to null}
  	p := StrEnd(bufptr)
	else
  	if p=bufptr then            {1st char is delim..."empty field"}
    	goto done;

      {Copy chars from bufptr up to next delim into FieldBuf}
  while (bufptr<p) do begin
    fp^ := bufptr^;
    Inc(fp);
    Inc(bufptr);
  end;

done:
			{if not already pointing to terminating null, pt to start of next fld}
	if bufptr^<>#0 then
  	Inc(bufptr);

reallydone:
	fp^ := #0;			{terminate field buffer}
end;


function TParseBuf.GetField: PChar;
{	Return pointer to first char of the field buffer, which is a null-terminated
	string.}
begin
	Result := @FieldBuf[0];
end;


procedure Register;
begin
  RegisterComponents('Data Access', [TParseBuf]);
end;


end.

